/**
 * EasyBeans
 * Copyright (C) 2006 Bull S.A.S.
 * Contact: easybeans@ow2.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * --------------------------------------------------------------------------
 * $Id: ASMScannerImpl.java 4389 2008-12-15 13:48:57Z alitokmen $
 * --------------------------------------------------------------------------
 */

package org.ow2.util.scan.impl;

import java.io.IOException;
import java.io.InputStream;
import java.net.URL;
import java.net.URLConnection;
import java.util.List;
import java.util.Set;

import org.ow2.util.asm.ClassReader;
import org.ow2.util.scan.api.IScanner;
import org.ow2.util.scan.api.ScanException;
import org.ow2.util.scan.api.configurator.IArchiveConfigurator;

/**
 * ArchiveScanner.
 * This scanner use ASM to analyze classes.
 * @author Gael Lalire
 */
public final class ASMScannerImpl implements IScanner {

    /**
     * scan the class contained in url.
     * @param url url of the class
     * @param referencedClasses this set is filled during analyze
     * @param archiveConfiguratorList configurators to call
     * @throws ScanException if error occurs
     */
    public void scanClass(final URL url, final Set<String> referencedClasses,
            final List<? extends IArchiveConfigurator> archiveConfiguratorList) throws ScanException {
        InputStream is = null;
        try {
            URLConnection urlConnection = url.openConnection();
            urlConnection.setDefaultUseCaches(false);
            is = urlConnection.getInputStream();
            new ClassReader(is).accept(new ScanClassVisitor(archiveConfiguratorList, referencedClasses), 0);
        } catch (Exception ioe) {
            throw new ScanException("Error while analyzing file entry '" + url + "'", ioe);
        } finally {
            if (is != null) {
                try {
                    is.close();
                } catch (IOException e) {
                    throw new ScanException("Error while closing input stream of the entry '" + url
                            + "'", e);
                }
            }
        }

    }

}
