/**
 * EasyBeans
 * Copyright (C) 2006 Bull S.A.S.
 * Contact: easybeans@ow2.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * --------------------------------------------------------------------------
 * $Id: ScanAnnotationVisitor.java 4389 2008-12-15 13:48:57Z alitokmen $
 * --------------------------------------------------------------------------
 */

package org.ow2.util.scan.impl;

import java.util.ArrayList;
import java.util.List;

import org.ow2.util.asm.AnnotationVisitor;
import org.ow2.util.asm.Type;
import org.ow2.util.scan.api.IAnnotationVisitor;

/**
 * ScanAnnotationVisitor.
 * @author Gael Lalire
 */
public final class ScanAnnotationVisitor implements AnnotationVisitor {

    /**
     * visitors.
     */
    private List<IAnnotationVisitor> annotationVisitors;

    /**
     * Constructor.
     * @param annotationVisitors visitors
     */
    public ScanAnnotationVisitor(final List<IAnnotationVisitor> annotationVisitors) {
        this.annotationVisitors = annotationVisitors;
    }

    /**
     * visit.
     * @param arg0 ?
     * @param arg1 ?
     */
    public void visit(final String arg0, final Object arg1) {
        Object value = arg1;
        if (value instanceof Type) {
            value = new TypeImpl((Type) value);
        }
        for (IAnnotationVisitor annotationVisitor : annotationVisitors) {
            if (annotationVisitor != null) {
                annotationVisitor.visit(arg0, value);
            }
        }
    }

    /**
     * visitAnnotation.
     * @param arg0 ?
     * @param arg1 ?
     * @return a scanner for all visitors
     */
    public AnnotationVisitor visitAnnotation(final String arg0, final String arg1) {
        List<IAnnotationVisitor> result = new ArrayList<IAnnotationVisitor>();
        for (IAnnotationVisitor annotationVisitor : annotationVisitors) {
            if (annotationVisitor != null) {
                result.add(annotationVisitor.visitAnnotation(arg0, arg1));
            }
        }
        return new ScanAnnotationVisitor(result);
    }

    /**
     * visitArray.
     * @param arg0 ?
     * @return a scanner for all visitors
     */
    public AnnotationVisitor visitArray(final String arg0) {
        List<IAnnotationVisitor> result = new ArrayList<IAnnotationVisitor>();
        for (IAnnotationVisitor annotationVisitor : annotationVisitors) {
            if (annotationVisitor != null) {
                result.add(annotationVisitor.visitArray(arg0));
            }
        }
        return new ScanAnnotationVisitor(result);
    }

    /**
     * visitEnd.
     */
    public void visitEnd() {
        for (IAnnotationVisitor annotationVisitor : annotationVisitors) {
            if (annotationVisitor != null) {
                annotationVisitor.visitEnd();
            }
        }
    }

    /**
     * visitEnum.
     * @param arg0 ?
     * @param arg1 ?
     * @param arg2 ?
     */
    public void visitEnum(final String arg0, final String arg1, final String arg2) {
        for (IAnnotationVisitor annotationVisitor : annotationVisitors) {
            if (annotationVisitor != null) {
                annotationVisitor.visitEnum(arg0, arg1, arg2);
            }
        }
    }

}
