/**
 * EasyBeans
 * Copyright (C) 2006 Bull S.A.S.
 * Contact: easybeans@ow2.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * --------------------------------------------------------------------------
 * $Id: ScanCommonVisitor.java 4389 2008-12-15 13:48:57Z alitokmen $
 * --------------------------------------------------------------------------
 */

package org.ow2.util.scan.impl;

import java.util.ArrayList;
import java.util.List;

import org.ow2.util.asm.AnnotationVisitor;
import org.ow2.util.asm.commons.EmptyVisitor;
import org.ow2.util.scan.api.IAnnotationVisitor;
import org.ow2.util.scan.api.configurator.ICommonConfigurator;
import org.ow2.util.scan.api.metadata.structures.JAnnotation;

/**
 * super class of all scanner.
 * @author Gael Lalire
 * @param <T> the class of configurator
 */
public class ScanCommonVisitor<T extends ICommonConfigurator> extends EmptyVisitor {

    /**
     * configurators.
     */
    private List<T> configurators;

    /**
     * @return configurators
     */
    protected List<T> getConfigurators() {
        return configurators;
    }

    /**
     * Set configurators.
     * @param configurators configurators
     */
    protected void setConfigurators(final List<T> configurators) {
        this.configurators = configurators;
    }

    /**
     * @param name annotation name
     * @param visible annotation visibility
     * @return a visitor
     */
    @Override
    public AnnotationVisitor visitAnnotation(final String name, final boolean visible) {
        JAnnotation jAnnotation = new JAnnotation(name, visible);
        List<IAnnotationVisitor> result = new ArrayList<IAnnotationVisitor>();

        for (T configurator : configurators) {
            IAnnotationVisitor annotationVisitor = configurator.getAnnotationVisitor(jAnnotation);
            result.add(annotationVisitor);
        }
        return new ScanAnnotationVisitor(result);
    }

}
