/**
 * EasyBeans
 * Copyright (C) 2006 Bull S.A.S.
 * Contact: easybeans@ow2.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * --------------------------------------------------------------------------
 * $Id: ScanMethodVisitor.java 4389 2008-12-15 13:48:57Z alitokmen $
 * --------------------------------------------------------------------------
 */

package org.ow2.util.scan.impl;

import java.util.List;
import java.util.Set;

import org.ow2.util.asm.MethodVisitor;
import org.ow2.util.scan.api.configurator.IMethodConfigurator;

/**
 * ScanMethodVisitor.
 * @author Gael Lalire
 */
public final class ScanMethodVisitor extends ScanCommonVisitor<IMethodConfigurator> implements MethodVisitor {

    /**
     * A set to complete with referenced classes name.
     */
    private Set<String> referencedClasses;

    /**
     * Constructor.
     * @param methodConfigurators configurators
     * @param referencedClasses a set to complete with referenced classes name.
     */
    public ScanMethodVisitor(final List<IMethodConfigurator> methodConfigurators, final Set<String> referencedClasses) {
        setConfigurators(methodConfigurators);
        this.referencedClasses = referencedClasses;
    }

    /**
     * call configurationCompleteon each configurator.
     */
    @Override
    public void visitEnd() {
        for (IMethodConfigurator methodConfigurator : getConfigurators()) {
            methodConfigurator.configurationComplete(referencedClasses);
        }
    }

}
